#include "tableprint.hpp"
#include <iomanip>
#include <algorithm>
#include <locale>
#include <cwchar>

tableprint::tableprint(const std::string title) : title(title) {
    // Set locale for wide character support
    std::setlocale(LC_ALL, "");
}

tableprint::~tableprint() {}

void tableprint::set_title(const std::string title) {
    this->title = title;
}

void tableprint::add_row(const std::vector<std::string>& row) {
    rows.push_back(row);
}

void tableprint::print() {
    if (rows.empty()) return;

    // Calculate column widths
    std::vector<size_t> col_widths(rows[0].size(), 0);
    for (const auto& row : rows) {
        for (size_t i = 0; i < row.size(); ++i) {
            col_widths[i] = std::max(col_widths[i], row[i].length());
        }
    }

    // Calculate total table width
    size_t total_width = 0;
    for (size_t width : col_widths) {
        total_width += width + 2; // +2 for padding
    }
    total_width += col_widths.size() - 1; // Add space for vertical borders

    // Print title if it exists
    if (!title.empty()) {
        std::wcout << L"\033[90m"; // Dark grey color for borders
        std::wcout << L"┌" << std::wstring(total_width, L'─') << L"┐" << std::endl;
        std::wcout << L"│" << L"\033[1;37m"; // White color for title
        size_t padding = (total_width - title.length()) / 2;
        std::wcout << std::wstring(padding, L' ') << std::wstring(title.begin(), title.end()) 
                  << std::wstring(total_width - title.length() - padding, L' ');
        std::wcout << L"\033[90m│" << std::endl;
        // Use └─┴─┘ for bottom of title box to connect with table
        std::wcout << L"├";
        for (size_t i = 0; i < rows[0].size(); ++i) {
            std::wcout << std::wstring(col_widths[i] + 2, L'─');
            if (i < rows[0].size() - 1) std::wcout << L"┬";
        }
        std::wcout << L"┤" << std::endl;
    } else {
        // Print top border if no title
        std::wcout << L"\033[90m"; // Dark grey color for borders
        std::wcout << L"┌";
        for (size_t i = 0; i < rows[0].size(); ++i) {
            std::wcout << std::wstring(col_widths[i] + 2, L'─');
            if (i < rows[0].size() - 1) std::wcout << L"┬";
        }
        std::wcout << L"┐" << std::endl;
    }

    // Print header
    std::wcout << L"│";
    std::wcout << L"\033[1;36m"; // Cyan color for header
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::wcout << L" " << std::setw(col_widths[i]) << std::left << std::wstring(rows[0][i].begin(), rows[0][i].end()) << L" ";
        if (i < rows[0].size() - 1) {
            std::wcout << L"\033[90m│\033[1;36m"; // Border color then back to cyan
        } else {
            std::wcout << L"\033[90m│"; // Just border color for last column
        }
    }
    std::wcout << std::endl;

    // Print header separator
    std::wcout << L"├";
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::wcout << std::wstring(col_widths[i] + 2, L'─');
        if (i < rows[0].size() - 1) std::wcout << L"┼";
    }
    std::wcout << L"┤" << std::endl;

    // Print rows
    for (size_t row_idx = 1; row_idx < rows.size(); ++row_idx) {
        const auto& row = rows[row_idx];
        std::wcout << L"│";
        for (size_t i = 0; i < row.size(); ++i) {
            std::string cell = row[i];
            // Replace :tick: and :cross: with colored symbols
            if (cell == ":tick:") {
                std::wcout << L" " << L"\033[1;32m" << std::setw(col_widths[i]) << std::left << L"✓" << L"\033[90m" << L" │";
            } else if (cell == ":cross:") {
                std::wcout << L" " << L"\033[1;31m" << std::setw(col_widths[i]) << std::left << L"✗" << L"\033[90m" << L" │";
            } else {
                std::wcout << L" " << L"\033[37m" << std::setw(col_widths[i]) << std::left << std::wstring(cell.begin(), cell.end()) << L"\033[90m" << L" │";
            }
        }
        std::wcout << std::endl;

        // Print row separator if not the last row
        if (row_idx < rows.size() - 1) {
            std::wcout << L"├";
            for (size_t i = 0; i < rows[0].size(); ++i) {
                std::wcout << std::wstring(col_widths[i] + 2, L'─');
                if (i < rows[0].size() - 1) std::wcout << L"┼";
            }
            std::wcout << L"┤" << std::endl;
        }
    }

    // Print bottom border
    std::wcout << L"└";
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::wcout << std::wstring(col_widths[i] + 2, L'─');
        if (i < rows[0].size() - 1) std::wcout << L"┴";
    }
    std::wcout << L"┘" << L"\033[0m" << std::endl; // Reset color
} 